//
//  ViewController.swift
//  Spatial
//
//  Created by Phidgets on 2018-01-31.
//  Copyright © 2018 Phidgets. All rights reserved.
//

import UIKit
import Phidget22Swift

class ViewController: UIViewController {

    let ch = Spatial()
    var phidgetInfoBox:PhidgetInfoBoxViewController! = nil
    
    @IBOutlet var accelerometerLabels: [UILabel]!
    @IBOutlet var gyroscopeLabels: [UILabel]!
    @IBOutlet var magnetometerLabels: [UILabel]!
    @IBOutlet var timestampLabel: UILabel!
    @IBOutlet var dataIntervalSlider: UISlider!
    @IBOutlet var dataIntervalLabel: UILabel!
    @IBOutlet var stackView: UIStackView!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        do {
            try Net.enableServerDiscovery(serverType: .deviceRemote)
            let _ = ch.attach.addHandler(attach_handler)
            let _ = ch.detach.addHandler(detach_handler)
            let _ = ch.error.addHandler(error_handler)
            let _ = ch.spatialData.addHandler(spatialdata_handler)
            try ch.open()
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    //MARK: Event Handlers
    func attach_handler(sender: Phidget){
        do{
            try phidgetInfoBox.fillPhidgetInfo(phid: sender)
            let attachedDevice = sender as! Spatial
            
            let minDataInterval = try Float(attachedDevice.getMinDataInterval())
            let maxDataInterval = try Float(attachedDevice.getMaxDataInterval())
            let dataInterval = try Float(attachedDevice.getDataInterval())
            
            
            DispatchQueue.main.async {
                self.stackView.isHidden = false
                self.dataIntervalSlider.minimumValue = minDataInterval
                self.dataIntervalSlider.maximumValue = maxDataInterval
                self.dataIntervalSlider.value = dataInterval
                self.dataIntervalLabel.text = String(format: "%d ms", UInt32(dataInterval))
            }
            
            
            
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
        
    }
    
    func detach_handler(sender: Phidget){
        do{
            try phidgetInfoBox.fillPhidgetInfo(phid: nil)
            DispatchQueue.main.async {
                self.stackView.isHidden = true
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    func error_handler(sender: Phidget, data: (code: ErrorEventCode, description: String)){
        self.outputErrorMessage(errorTitle: "Error Code: \(data.code)", errorMessage: data.description)
    }
    var timeStamp:TimeInterval = 0
    var lastTimeStamp:TimeInterval = 0
    func outputErrorMessage(errorTitle:String, errorMessage:String){
        timeStamp = NSDate.timeIntervalSinceReferenceDate
        //Add a delay for error events so the user does not get bombarded by error events
        if(timeStamp - lastTimeStamp > 5){
            let alert = UIAlertController(title: errorTitle, message: errorMessage, preferredStyle: .alert)
            alert.addAction(UIAlertAction(title: "OK", style: .default, handler: {action -> Void in
            }))
            self.present(alert, animated: true, completion: nil)
            lastTimeStamp = timeStamp
        }
    }
    
    func spatialdata_handler(sender: Spatial, data: (acceleration: [Double], angularRate: [Double], magneticField: [Double], timestamp: Double)){
        DispatchQueue.main.async {
            for i in 0 ..< 3{
                self.accelerometerLabels[i].text = String(format: "%0.3f g", data.acceleration[i])
                self.gyroscopeLabels[i].text = String(format: "%0.3f °/s", data.angularRate[i])
                self.magnetometerLabels[i].text = String(format: "%0.3f G", data.magneticField[i])
            }
            self.timestampLabel.text = String(format:"%.0f ms",data.timestamp)
        }
    }

    //MARK: GUI controls
    @IBAction func setDataInterval(_ sender: Any) {
        let slider = sender as! UISlider
        do{
            try ch.setDataInterval(UInt32(slider.value))
            DispatchQueue.main.async {
                self.dataIntervalLabel.text = "\(UInt32(slider.value)) ms"
            }
            
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    //MARK: navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if segue.identifier == "infoBoxSegue" {
            phidgetInfoBox = segue.destination as! PhidgetInfoBoxViewController
        }
    }
    
    //MARK: Misc
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
    }
}

